# -*- coding: utf-8 -*-

# Copyright(C) 2012-2013 Romain Bignon
#
# This file is part of a woob module.
#
# This woob module is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This woob module is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this woob module. If not, see <http://www.gnu.org/licenses/>.

# flake8: compatible

from __future__ import unicode_literals

from .compat.woob_capabilities_bank import AccountNotFound
from woob.capabilities.bank.wealth import CapBankWealth
from .compat.woob_capabilities_base import find_object
from woob.tools.backend import Module, BackendConfig
from woob.tools.value import ValueBackendPassword, ValueTransient
from woob.capabilities.profile import CapProfile

from .browser import HSBC


__all__ = ['HSBCModule']


class HSBCModule(Module, CapBankWealth, CapProfile):
    NAME = 'hsbc'
    MAINTAINER = 'Romain Bignon'
    EMAIL = 'romain@weboob.org'
    VERSION = '3.0'
    LICENSE = 'LGPLv3+'
    DESCRIPTION = 'HSBC France'
    CONFIG = BackendConfig(
        ValueBackendPassword('login', label='Identifiant', regexp=r'^\d{11}$', masked=False),
        ValueBackendPassword('password', label='Mot de passe'),
        ValueBackendPassword('secret', label=u'Réponse secrète'),
        ValueTransient('otp'),
        ValueTransient('request_information'),
    )
    BROWSER = HSBC

    def create_default_browser(self):
        return self.create_browser(
            self.config,
            self.config['login'].get(),
            self.config['password'].get(),
            self.config['secret'].get()
        )

    def iter_accounts(self):
        for account in self.browser.iter_account_owners():
            yield account

    def get_account(self, _id):
        return find_object(self.browser.iter_account_owners(), id=_id, error=AccountNotFound)

    def iter_history(self, account):
        for tr in self.browser.get_history(account):
            yield tr

    def iter_investment(self, account):
        for tr in self.browser.get_investments(account):
            yield tr

    def iter_coming(self, account):
        for tr in self.browser.get_history(account, coming=True):
            yield tr

    def get_profile(self):
        return self.browser.get_profile()
